# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

* [`cisco_ios`](#cisco_ios): This class calls the cisco_ios::install class.
* [`cisco_ios::install`](#cisco_iosinstall): This class installs dependencies of this module
into the puppet agent, and/or the puppetserver service.
* [`cisco_ios::install::agent`](#cisco_iosinstallagent): This class install dependencies of this module into puppet agent
* [`cisco_ios::install::server`](#cisco_iosinstallserver): This class installs dependencies of this module into puppetserver,
and restarts the puppetserver service to activate.
* [`cisco_ios::proxy`](#cisco_iosproxy): This class install dependencies of this module into puppet agent
* [`cisco_ios::server`](#cisco_iosserver): This class installs dependencies of this module into puppetserver,
and restarts the puppetserver service to activate.

**Resource types**

* [`ios_aaa_accounting`](#ios_aaa_accounting): Configure aaa accounting on device
* [`ios_aaa_authentication`](#ios_aaa_authentication): Configure aaa authentication on device
* [`ios_aaa_authorization`](#ios_aaa_authorization): Configure aaa authorization on device
* [`ios_aaa_new_model`](#ios_aaa_new_model): Enable aaa new model on device
* [`ios_aaa_session_id`](#ios_aaa_session_id): Configure aaa session id on device
* [`ios_access_list`](#ios_access_list): Configure access lists. Deprecated, due to unreconcilable implementation issues. Use the ios_acl type instead.
* [`ios_acl`](#ios_acl): Manage ACL contents
* [`ios_acl_entry`](#ios_acl_entry): Configure access lists entries. Deprecated, due to unreconcilable implementation issues. Use the ios_acl type instead.
* [`ios_additional_syslog_settings`](#ios_additional_syslog_settings): Configure global syslog settings
* [`ios_cef`](#ios_cef): Implementation and Management of CEF settings
* [`ios_config`](#ios_config): Execute an arbitary configuration against the cisco_ios device with or without a check for idempotency
* [`ios_interface`](#ios_interface): Manage layer 3 configuration on a per Instance basis.
* [`ios_ip`](#ios_ip): Manages settings related to the IP
* [`ios_network_dns`](#ios_network_dns): Configure DNS settings for network devices
* [`ios_network_trunk`](#ios_network_trunk): Ethernet logical (switch-port) interface.  Configures VLAN trunking.
* [`ios_ntp_access_group`](#ios_ntp_access_group): Specify NTP access group config
* [`ios_ntp_config`](#ios_ntp_config): Specify NTP config
* [`ios_radius_global`](#ios_radius_global): Configure IOS global RADIUS settings
* [`ios_radius_server_group`](#ios_radius_server_group): Configure a RADIUS server group
* [`ios_snmp_global`](#ios_snmp_global): Configures Global snmp settings.
* [`ios_stp_global`](#ios_stp_global): Manages the Cisco Spanning-tree Global configuration resource.

**Tasks**

* [`cli_command`](#cli_command): Execute CLI Command
* [`config_save`](#config_save): Save running-config to startup-config
* [`restore_startup`](#restore_startup): Copies the startup-config to the running-config
* [`backup_config`](#backup_config): Backs up the running config on the device to a given file location
* [`restore_config`](#restore_config): Restores the config from the specified location to the device

## Classes

### cisco_ios

This class calls the cisco_ios::install class.

#### Examples

##### Declaring the class

```puppet
include cisco_ios
```

### cisco_ios::install

This class installs dependencies of this module
into the puppet agent, and/or the puppetserver service.

#### Examples

##### Declaring the class

```puppet
include cisco_ios::install
```

### cisco_ios::install::agent

This class install dependencies of this module into puppet agent

#### Examples

##### Declaring the class

```puppet
include cisco_ios::install::agent
```

### cisco_ios::install::server

This class installs dependencies of this module into puppetserver,
and restarts the puppetserver service to activate.

#### Examples

##### Declaring the class

```puppet
include cisco_ios::install::server
```

### cisco_ios::proxy

This class install dependencies of this module into puppet agent

* **Note** Deprecated, use cisco_ios::install::agent

#### Examples

##### Declaring the class

```puppet
include cisco_ios::proxy
```

### cisco_ios::server

This class installs dependencies of this module into puppetserver,
and restarts the puppetserver service to activate.

* **Note** Deprecated, use cisco_ios::install::server

#### Examples

##### Declaring the class

```puppet
include cisco_ios::server
```

## Resource types

### ios_aaa_accounting

Configure aaa accounting on device

#### Properties

The following properties are available in the `ios_aaa_accounting` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this aaa accounting should be present or absent on the target system.

Default value: present

##### `accounting_service`

Data type: `Enum["auth-proxy","commands","connection","dot1x","exec","identity","network","onep","resource","system","update"]`

AAA Accounting service to use

##### `commands_enable_level`

Data type: `Optional[Integer]`

Enable level - needed for "commands" accounting_service

##### `accounting_list`

Data type: `Optional[String]`

The accounting list - named or default

##### `accounting_status`

Data type: `Optional[Enum["none","start-stop","stop-only"]]`

The status of the accounting

##### `server_groups`

Data type: `Optional[Array[String]]`

Array of the server groups eg. `['tacacs+'], ['test1', 'test2']`

##### `update_newinfo`

Data type: `Optional[Boolean]`

Only send accounting update records when we have new acct info. (For periodic use "update_newinfo_periodic") - use with "update" accounting_service.

##### `update_newinfo_periodic`

Data type: `Optional[Integer[1, 2147483647]]`

Periodic intervals to send accounting update records(in minutes) when we have new acct info. (For non-periodic use "update_newinfo")  - use with "update" accounting_service.

##### `update_periodic`

Data type: `Optional[Integer[1, 2147483647]]`

Periodic intervals to send accounting update records(in minutes) (For new acct info only use "update_newinfo_periodic") - use with "update" accounting_service.

#### Parameters

The following parameters are available in the `ios_aaa_accounting` type.

##### `name`

namevar

Data type: `String`

Name. On resource this is a composite of the authorization_service (and enable level if "commands")
 and authorization_list name eg. "commands 15 default" or "exec authlist1" - or "update" type eg. "update newinfo"

Default value: default

### ios_aaa_authentication

Configure aaa authentication on device

#### Properties

The following properties are available in the `ios_aaa_authentication` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this aaa authentication should be present or absent on the target system.

Default value: present

##### `authentication_list_set`

Data type: `Enum["arap","login","enable","dot1x","eou","onep","ppp","sgbp","suppress"]`

Set authentication lists for - Login, Enable or dot1x

##### `authentication_list`

Data type: `Optional[String]`

The authentication list - named or default

##### `cache_groups`

Data type: `Optional[Array[String]]`

Array of the cache groups eg. ['ldap', 'radius', tacacs+'], ['test1', 'test2']

##### `server_groups`

Data type: `Optional[Array[String]]`

Array of the server groups eg. ['tacacs+'], `['test1', 'test2']`

##### `enable_password`

Data type: `Optional[Boolean]`

Use enable password for authentication.

##### `line`

Data type: `Optional[Boolean]`

Use line password for authentication.

##### `local`

Data type: `Optional[Boolean]`

Use local username authentication.

##### `local_case`

Data type: `Optional[Boolean]`

Use case-sensitive local username authentication.

##### `no_authentication`

Data type: `Optional[Boolean]`

NO authentication.

##### `switch_auth`

Data type: `Optional[Boolean]`

Switch authentication.

##### `suppress_null_username`

Data type: `Optional[Boolean]`

Do not send access-request for users with null username.

#### Parameters

The following parameters are available in the `ios_aaa_authentication` type.

##### `name`

namevar

Data type: `String`

Name. On resource this is a composite of the authentication_list_set and authentication_list name eg. "login default"

Default value: default

### ios_aaa_authorization

Configure aaa authorization on device

#### Properties

The following properties are available in the `ios_aaa_authorization` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this aaa authorization should be present or absent on the target system.

Default value: present

##### `authorization_service`

Data type: `Enum["auth-proxy","commands","configuration","exec","network","reverse_access"]`

AAA Authorization service to use

##### `commands_enable_level`

Data type: `Optional[Integer]`

Enable level - needed for "commands" authorization_service

##### `authorization_list`

Data type: `String`

The authorization list - named or default

Default value: default

##### `server_groups`

Data type: `Optional[Array[String]]`

Array of the server groups eg. ['tacacs+'], ['test1', 'test2']

##### `local`

Data type: `Optional[Boolean]`

Use local database.

##### `if_authenticated`

Data type: `Optional[Boolean]`

Succeed if user has authenticated.

#### Parameters

The following parameters are available in the `ios_aaa_authorization` type.

##### `name`

namevar

Data type: `String`

Name. On resource this is a composite of the authorization_service (and enable level if "commands") and authorization_list name eg. "commands 15 default" or "exec authlist1"

Default value: default

### ios_aaa_new_model

Enable aaa new model on device

#### Properties

The following properties are available in the `ios_aaa_new_model` type.

##### `enable`

Data type: `Boolean`

Enable or disable aaa new model

#### Parameters

The following parameters are available in the `ios_aaa_new_model` type.

##### `name`

namevar

Data type: `String`

The name stays as "default"

Default value: default

### ios_aaa_session_id

Configure aaa session id on device

#### Properties

The following properties are available in the `ios_aaa_session_id` type.

##### `session_id_type`

Data type: `Enum["common","unique"]`

Type of aaa session id - common or unique

#### Parameters

The following parameters are available in the `ios_aaa_session_id` type.

##### `name`

namevar

Data type: `String`

The name stays as "default"

Default value: default

### ios_access_list

Configure access lists. Deprecated, due to unreconcilable implementation issues. Use the ios_acl type instead.

#### Properties

The following properties are available in the `ios_access_list` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this access list should be present or absent on the target system.

Default value: present

##### `access_list_type`

Data type: `Enum["Standard","Extended","Reflexive","none"]`

Type of access list - standard, extended, reflexive or no type

#### Parameters

The following parameters are available in the `ios_access_list` type.

##### `name`

namevar

Data type: `String`

Access list name or number.

### ios_acl

Manage ACL contents

#### Properties

The following properties are available in the `ios_acl` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this access list entry should be present or absent on the target system.

Default value: present

##### `dynamic`

Data type: `Optional[String]`

Name of a Dynamic list

##### `permission`

Data type: `Enum["permit", "deny", "evaluate"]`

Specify packets to forward/reject, or evaluate an access list

##### `evaluation_name`

Data type: `Optional[String]`

Evaluate an access list

##### `protocol`

Data type: `Optional[Variant[Enum["ahp","eigrp","esp","gre","icmp","igmp","ip","ipinip","nos","ospf","pcp","pim","tcp","udp"],Pattern[/\d+/]]]`
_*this data type contains a regex that may not be accurately reflected in generated documentation_

ACL Entry Protocol

##### `source_address`

Data type: `Optional[String]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_group`

Data type: `Optional[String]`

Source Address object-group. Either Source Address, address object-group, any or source host are required.

##### `source_address_any`

Data type: `Optional[Boolean]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_host`

Data type: `Optional[String]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_wildcard_mask`

Data type: `Optional[String]`

Source Address wildcard mask. Must be used with, and only used with, Source Address.

##### `source_eq`

Data type: `Optional[Array[String]]`

Match only packets on a given port number.

##### `source_gt`

Data type: `Optional[String]`

Match only packets with a greater port number.

##### `source_lt`

Data type: `Optional[String]`

Match only packets with a lower port number.

##### `source_neq`

Data type: `Optional[String]`

Match only packets not on a given port number.

##### `source_portgroup`

Data type: `Optional[String]`

Destination port object-group.

##### `source_range`

Data type: `Optional[Array[String]]`

Match only packets in the range of port numbers.

##### `destination_address`

Data type: `Optional[String]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_group`

Data type: `Optional[String]`

Destination Address object-group. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_any`

Data type: `Optional[Boolean]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_host`

Data type: `Optional[String]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_wildcard_mask`

Data type: `Optional[String]`

Destination Address wildcard mask. Must be used with, and only used with, Destination Address.

##### `destination_eq`

Data type: `Optional[Array[String]]`

Match only packets on a given port number.

##### `destination_gt`

Data type: `Optional[String]`

Match only packets with a greater port number.

##### `destination_lt`

Data type: `Optional[String]`

Match only packets with a lower port number.

##### `destination_neq`

Data type: `Optional[String]`

Match only packets not on a given port number.

##### `destination_portgroup`

Data type: `Optional[String]`

Destination port object-group.

##### `destination_range`

Data type: `Optional[Array[String]]`

Match only packets in the range of port numbers.

##### `ack`

Data type: `Optional[Boolean]`

Match on the ACK bit.

##### `dscp`

Data type: `Optional[String]`

Match packets with given dscp value.

##### `fin`

Data type: `Optional[Boolean]`

Match on the FIN bit.

##### `fragments`

Data type: `Optional[Boolean]`

Check non-initial fragments.

##### `icmp_message_code`

Data type: `Optional[Integer]`

ICMP message code.

##### `icmp_message_type`

Data type: `Optional[Variant[String, Integer]]`

ICMP message type.

##### `igmp_message_type`

Data type: `Optional[Variant[String, Integer]]`

IGMP message type.

##### `log`

Data type: `Optional[Boolean]`

Log matches against this entry. Either log or log_input can be used, but not both.

##### `log_input`

Data type: `Optional[Boolean]`

Log matches against this entry, including input interface. Either log or log_input can be used, but not both.

##### `match_all`

Data type: `Optional[Array[String]]`

Match if all specified flags are present.

##### `match_any`

Data type: `Optional[Array[String]]`

Match if any specified flags are present.

##### `option`

Data type: `Optional[String]`

Match packets with given IP Options value.

##### `precedence`

Data type: `Optional[String]`

Match packets with given precedence value.

##### `psh`

Data type: `Optional[Boolean]`

Match on the PSH bit.

##### `reflect`

Data type: `Optional[String]`

Create reflexive access list entry.

##### `reflect_timeout`

Data type: `Optional[Integer]`

Maximum time to live in seconds. Only to be used with reflect.

##### `rst`

Data type: `Optional[Boolean]`

Match on the RST bit.

##### `syn`

Data type: `Optional[Boolean]`

Match on the SYN bit.

##### `time_range`

Data type: `Optional[String]`

Specify a time-range.

##### `tos`

Data type: `Optional[String]`

Match packets with given TOS value.

##### `urg`

Data type: `Optional[Boolean]`

Match on the URG bit.

#### Parameters

The following parameters are available in the `ios_acl` type.

##### `access_list`

namevar

Data type: `String`

Name of parent access list

##### `entry`

namevar

Data type: `String`

Name. Used as sequence number <1-2147483647>

##### `access_list_type`

namevar

Data type: `Enum["standard","extended","reflexive","none"]`

Type of access list - standard, extended, reflexive or no type

### ios_acl_entry

Configure access lists entries. Deprecated, due to unreconcilable implementation issues. Use the ios_acl type instead.

#### Properties

The following properties are available in the `ios_acl_entry` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this access list entry should be present or absent on the target system.

Default value: present

##### `access_list`

Data type: `String`

Name of parent access list

##### `entry`

Data type: `Integer`

Name. Used as sequence number <1-2147483647>

##### `dynamic`

Data type: `Optional[String]`

Name of a Dynamic list

##### `permission`

Data type: `Enum["permit", "deny", "evaluate"]`

Specify packets to forward/reject, or evaluate an access list

##### `evaluation_name`

Data type: `Optional[String]`

Evaluate an access list

##### `protocol`

Data type: `Optional[Variant[Enum["ahp","eigrp","esp","gre","icmp","igmp","ip","ipinip","nos","ospf","pcp","pim","tcp","udp"],Pattern[/\d+/]]]`
_*this data type contains a regex that may not be accurately reflected in generated documentation_

The ACL Entry Protocol

Example:
```puppet
protocol => 'tcp'
```
```puppet
protocol => '8'
```

##### `source_address`

Data type: `Optional[String]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_group`

Data type: `Optional[String]`

Source Address object-group. Either Source Address, address object-group, any or source host are required.

##### `source_address_any`

Data type: `Optional[Boolean]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_host`

Data type: `Optional[String]`

Source Address. Either Source Address, address object-group, any or source host are required.

##### `source_address_wildcard_mask`

Data type: `Optional[String]`

Source Address wildcard mask. Must be used with, and only used with, Source Address.

##### `source_eq`

Data type: `Optional[Array[String]]`

Match only packets on a given port number.

##### `source_gt`

Data type: `Optional[String]`

Match only packets with a greater port number.

##### `source_lt`

Data type: `Optional[String]`

Match only packets with a lower port number.

##### `source_neq`

Data type: `Optional[String]`

Match only packets not on a given port number.

##### `source_portgroup`

Data type: `Optional[String]`

Destination port object-group.

##### `source_range`

Data type: `Optional[Array[String]]`

Match only packets in the range of port numbers.

##### `destination_address`

Data type: `Optional[String]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_group`

Data type: `Optional[String]`

Destination Address object-group. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_any`

Data type: `Optional[Boolean]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_host`

Data type: `Optional[String]`

Destination Address. Either Destination Address, address object-group, any or destination host are required.

##### `destination_address_wildcard_mask`

Data type: `Optional[String]`

Destination Address wildcard mask. Must be used with, and only used with, Destination Address.

##### `destination_eq`

Data type: `Optional[Array[String]]`

Match only packets on a given port number.

##### `destination_gt`

Data type: `Optional[String]`

Match only packets with a greater port number.

##### `destination_lt`

Data type: `Optional[String]`

Match only packets with a lower port number.

##### `destination_neq`

Data type: `Optional[String]`

Match only packets not on a given port number.

##### `destination_portgroup`

Data type: `Optional[String]`

Destination port object-group.

##### `destination_range`

Data type: `Optional[Array[String]]`

Match only packets in the range of port numbers.

##### `ack`

Data type: `Optional[Boolean]`

Match on the ACK bit.

##### `dscp`

Data type: `Optional[String]`

Match packets with given dscp value.

##### `fin`

Data type: `Optional[Boolean]`

Match on the FIN bit.

##### `fragments`

Data type: `Optional[Boolean]`

Check non-initial fragments.

##### `icmp_message_code`

Data type: `Optional[Integer]`

ICMP message code.

##### `icmp_message_type`

Data type: `Optional[Variant[String, Integer]]`

ICMP message type.

##### `igmp_message_type`

Data type: `Optional[Variant[String, Integer]]`

IGMP message type.

##### `log`

Data type: `Optional[Boolean]`

Log matches against this entry. Either log or log_input can be used, but not both.

##### `log_input`

Data type: `Optional[Boolean]`

Log matches against this entry, including input interface. Either log or log_input can be used, but not both.

##### `match_all`

Data type: `Optional[Array[String]]`

Match if all specified flags are present.

##### `match_any`

Data type: `Optional[Array[String]]`

Match if any specified flags are present.

##### `option`

Data type: `Optional[String]`

Match packets with given IP Options value.

##### `precedence`

Data type: `Optional[String]`

Match packets with given precedence value.

##### `psh`

Data type: `Optional[Boolean]`

Match on the PSH bit.

##### `reflect`

Data type: `Optional[String]`

Create reflexive access list entry.

##### `reflect_timeout`

Data type: `Optional[Integer]`

Maximum time to live in seconds. Only to be used with reflect.

##### `rst`

Data type: `Optional[Boolean]`

Match on the RST bit.

##### `syn`

Data type: `Optional[Boolean]`

Match on the SYN bit.

##### `time_range`

Data type: `Optional[String]`

Specify a time-range.

##### `tos`

Data type: `Optional[String]`

Match packets with given TOS value.

##### `urg`

Data type: `Optional[Boolean]`

Match on the URG bit.

#### Parameters

The following parameters are available in the `ios_acl_entry` type.

##### `name`

namevar

Data type: `String`

Name. Made up of access_list and the entry with a space seperator. eg. "list42 10" is from access_list list42 and entry 10.

### ios_additional_syslog_settings

Configure global syslog settings

#### Properties

The following properties are available in the `ios_additional_syslog_settings` type.

##### `trap`

Data type: `Optional[Variant[Integer[0,7], Enum["unset"]]]`

Set the syslog server logging level, can be set to a severity level of [0-7] or 'unset'.

Examples:

```Puppet
  trap => 3,
```

```Puppet
  trap => 'unset',
```

##### `origin_id`

Data type: `Optional[Variant[Enum['hostname', 'ip', 'ipv6', unset], Tuple[Enum['string'], String]]]`

Sets an origin-id to be added to all syslog messages, can be set to a default value taken from the switch itself or a designated one word string.

Examples:

```Puppet
  origin_id => 'ipv6',
```

```Puppet
  origin_id => ['string', 'Main'],
```

```Puppet
  origin_id => 'unset',
```

#### Parameters

The following parameters are available in the `ios_additional_syslog_settings` type.

##### `name`

namevar

Data type: `String`

Name, generally "default", not used to manage the resource

Default value: default

### ios_cef

Implementation and Management of CEF settings

#### Properties

The following properties are available in the `ios_cef` type.

##### `distributed`

Data type: `Optional[Boolean]`

Distributed Cisco Express Forwarding

##### `optimize_resolution`

Data type: `Optional[Boolean]`

Trigger layer 2 address resolution directly from CEF

##### `load_sharing`

Data type: `Optional[Variant[Enum['original', 'tunnel', 'universal'], Tuple[Enum['tunnel', 'universal'], String], Tuple[Enum['include-ports'], Enum['destination', 'source']], Tuple[Enum['include-ports'], Enum['destination', 'source'], String]]]`

Per-destination load sharing algorithm selection

#### Parameters

The following parameters are available in the `ios_cef` type.

##### `name`

namevar

Data type: `String`

Resource name, not used to manage the device

Default value: default

### ios_config

Execute an arbitary configuration against the cisco_ios device with or without a check for idempotency

#### Properties

The following properties are available in the `ios_config` type.

##### `command`

Data type: `String`

The ios command to run

#### Parameters

The following parameters are available in the `ios_config` type.

##### `name`

namevar

Data type: `String`

The friendly name for this ios command

##### `command_mode`

Data type: `Optional[Enum["CONF_T"]]`

The command line mode to be in, when executing the command

Default value: CONF_T

##### `idempotent_regex`

Data type: `Optional[String]`

Expected string, when running a regex against the 'show running-config'

##### `idempotent_regex_options`

Data type: `Optional[Array[Enum["ignorecase","extended","multiline","fixedencoding","noencoding"]]]`

Array of one or more options which control how the pattern can match.

##### `negate_idempotent_regex`

Data type: `Optional[Boolean]`

Negate the regex used with idempotent_regex

### ios_interface

Manage layer 3 configuration on a per Instance basis.

#### Properties

The following properties are available in the `ios_interface` type.

##### `mac_notification_added`

Data type: `Optional[Boolean]`

Whether to enable Mac Address added notification for this port.

##### `mac_notification_removed`

Data type: `Optional[Boolean]`

Whether to enable Mac Address removed notification for this port.

##### `link_status_duplicates`

Data type: `Optional[Boolean]`

Whether to permit duplicate SNMP LINKUP and LINKDOWN traps.

##### `logging_event`

Data type: `Optional[Variant[Enum["unset"], Array[Enum["bundle-status","nfas-status","spanning-tree","status","subif-link-status","trunk-status","power-inline-status"]]]]`

Whether or not to log certain event messages. Any event log not specifically indicated will be disabled.

Example:
```puppet
  logging_event => ['spanning-tree','subif-link-status'],
```
```puppet
  logging_event => 'unset',
```

##### `logging_event_link_status`

Data type: `Optional[Boolean]`

Whether to log UPDOWN and CHANGE event messages.

##### `ip_dhcp_snooping_trust`

Data type: `Optional[Boolean]`

DHCP Snooping trust config.

##### `ip_dhcp_snooping_limit`

Data type: `Optional[Variant[Boolean[false], Integer[1, 2048]]]`

DHCP snooping rate limit.

Example:
```puppet
  ip_dhcp_snooping_limit => 1500,
```
```puppet
  ip_dhcp_snooping_limit => false,
```

##### `flowcontrol_receive`

Data type: `Optional[Enum["desired","on","off"]]`

Flow control (receive) [desired|on|off]

Example:
```puppet
  flowcontrol_receive => 'desired',
```

##### `vrf`

Data type: `Optional[String]`

VRF config

##### `route_cache_cef`

Data type: `Optional[Boolean]`

Enable/Disable Cisco Express Forwarding

#### Parameters

The following parameters are available in the `ios_interface` type.

##### `name`

namevar

Data type: `String`

The switch interface name, e.g. "Ethernet1"

### ios_ip

Manages settings related to the IP

#### Properties

The following properties are available in the `ios_ip` type.

##### `routing`

Data type: `Optional[Boolean]`

Whether to Enable IP routing

#### Parameters

The following parameters are available in the `ios_ip` type.

##### `name`

namevar

Data type: `String`

Resource name, not used to manage the device

Default value: default

### ios_network_dns

Configure DNS settings for network devices

#### Properties

The following properties are available in the `ios_network_dns` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether the network dns should be present or absent on the target system.

Default value: present

##### `domain`

Data type: `Optional[String]`

The default domain name to append to the device hostname

##### `hostname`

Data type: `Optional[String]`

The host name of the device

##### `search`

Data type: `Optional[Array[String]]`

Array of DNS suffixes to search for FQDN entries

##### `servers`

Data type: `Optional[Array[String]]`

Array of DNS servers to use for name resolution

##### `ip_domain_lookup`

Data type: `Optional[Boolean]`

Sets whether the Domain Name Server (DNS) lookup feature should be enabled

#### Parameters

The following parameters are available in the `ios_network_dns` type.

##### `name`

namevar

Data type: `String`

Name, generally "settings", not used to manage the resource

Default value: settings

### ios_network_trunk

Ethernet logical (switch-port) interface.  Configures VLAN trunking.

#### Properties

The following properties are available in the `ios_network_trunk` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether the network_trunk should be present or absent on the target system.

Default value: present

##### `encapsulation`

Data type: `Optional[Enum["dot1q","isl","negotiate","none"]]`

The vlan-tagging encapsulation protocol, usually dot1q

##### `mode`

Data type: `Optional[Enum["access","trunk","dynamic_auto","dynamic_desirable"]]`

The L2 interface mode, enables or disables trunking

##### `untagged_vlan`

Data type: `Optional[Integer[0, 4095]]`

VLAN used for untagged VLAN traffic. a.k.a Native VLAN

##### `tagged_vlans`

Data type: `Optional[Array[String]]`

Array of VLAN names used for tagged packets

##### `pruned_vlans`

Data type: `Optional[Array[String]]`

Array of VLAN ID numbers used for VLAN pruning

##### `access_vlan`

Data type: `Optional[Variant[Integer[0, 4095], Boolean[false]]]`

The VLAN to set when the interface is in access mode. Setting it to false will revert it to the default value.

Examples:

```Puppet
access_vlan => 405
```

```Puppet
access_vlan => false
```

##### `voice_vlan`

Data type: `Optional[Variant[Integer[0, 4095], Enum["dot1p", "none", "untagged"], Boolean[false]]]`

Sets how voice traffic should be treated by the access port. Setting it to false will revert it to the default value.

Examples:

```Puppet
access_vlan => 221
```

```Puppet
access_vlan => 'dot1p'
```

```Puppet
access_vlan => 'false'
```

##### `switchport_nonegotiate`

Data type: `Optional[Boolean]`

When set, prevents the port from sending DTP (Dynamic Trunk Port) messages. Set automatically to true while in 'access mode' and cannot be set in 'dynamic_*' mode.

Examples:

```Puppet
access_vlan => true
```

See `network_trunk` for other availible fields.

##### `allowed_vlans`

Data type: `Optional[Variant[Enum["all", "none"], Tuple[Enum["add", "remove", "except"], String], String, Boolean[false]]]`

Sets which VLANs the access port will use when trunking is enabled. Setting it to false will revert it to the default value.

Examples:

```Puppet
access_vlan => '101-202'
```

```Puppet
access_vlan => 'none'
```

```Puppet
access_vlan => ['except', '204-301']
```

#### Parameters

The following parameters are available in the `ios_network_trunk` type.

##### `name`

namevar

Data type: `String`

The switch interface name, e.g. "Ethernet1"

### ios_ntp_access_group

Specify NTP access group config

#### Properties

The following properties are available in the `ios_ntp_access_group` type.

##### `access_group_type`

Data type: `Enum["peer", "serve", "query-only", "serve-only"]`

Defines the access group type

##### `ipv6_access_group`

Data type: `Optional[Boolean]`

Whether this access group makes use of ipv6

##### `ensure`

Data type: `Enum[present, absent]`

Whether this access group entry should be present or absent on the target system.

Default value: present

#### Parameters

The following parameters are available in the `ios_ntp_access_group` type.

##### `name`

namevar

Data type: `String`

Configuration name, ip access list name

### ios_ntp_config

Specify NTP config

#### Properties

The following properties are available in the `ios_ntp_config` type.

##### `authenticate`

Data type: `Optional[Boolean]`

NTP authentication enabled [true|false]

##### `source_interface`

Data type: `Optional[String]`

The source interface for the NTP system

##### `trusted_key`

Data type: `Optional[Array[Variant[Integer, String]]]`

Array of global trusted-keys. Contents can be a String or Integers

##### `update_calendar`

Data type: `Optional[Boolean]`

Whether the update calendar option is enabled on the system

##### `logging`

Data type: `Optional[Boolean]`

Whether to enable NTP message logging

#### Parameters

The following parameters are available in the `ios_ntp_config` type.

##### `name`

namevar

Data type: `String`

Config name, default to "default" as the NTP config is global rather than instance based

Default value: default

### ios_radius_global

Configure IOS global RADIUS settings

#### Properties

The following properties are available in the `ios_radius_global` type.

##### `enable`

Data type: `Optional[Boolean]`

Enable or disable RADIUS functionality [true|false]

##### `attributes`

Data type: `Optional[Array[Tuple[Integer, String]]]`

An array of [attribute number, attribute options] pairs,

> NOTE: There are a huge number of attributes available across devices with varying configuration options. Some of these pose issues for idempotency.
>
> This modules does not attempt to solve these issues and you should take care to review your settings.
>
> Example:
>
> `[11, 'default direction inbound']` will set correctly, however the device will return `[11, 'default direction in']`. You should prefer setting `[11, 'default direction in']`
>
> Example:
>
> `[11, 'default direction outbound']` will set correctly, however the device will remove the setting from the config as this is a default. You should instead prefer not setting this option.


##### `key`

Data type: `Optional[String]`

Encryption key (plaintext or in hash form depending on key_format)

##### `key_format`

Data type: `Optional[Integer]`

Encryption key format [0-7]

##### `retransmit_count`

Data type: `Optional[Variant[Integer, Enum["unset"]]]`

How many times to retransmit or 'unset'

##### `source_interface`

Data type: `Optional[Array[String]]`

The source interface used for RADIUS packets (array of strings for multiple).

##### `timeout`

Data type: `Optional[Variant[Integer, Enum["unset"]]]`

Number of seconds before the timeout period ends or 'unset'

##### `vrf`

Data type: `Optional[Array[String]]`

The VRF associated with source_interface (array of strings for multiple).

#### Parameters

The following parameters are available in the `ios_radius_global` type.

##### `name`

namevar

Data type: `String`

Resource name, not used to manage the device

Default value: default

### ios_radius_server_group

Configure a RADIUS server group

#### Properties

The following properties are available in the `ios_radius_server_group` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether radius_server_group should be present or absent on the target system.

Default value: present

##### `servers`

Data type: `Optional[Array[String]]`

Array of DNS suffixes to search for FQDN entries

##### `private_servers`

Data type: `Optional[Array[String]]`

Array of private DNS suffixes to search for FQDN entries

#### Parameters

The following parameters are available in the `ios_radius_server_group` type.

##### `name`

namevar

Data type: `String`

The name of the RADIUS server group

### ios_snmp_global

Configures Global snmp settings.

#### Properties

The following properties are available in the `ios_snmp_global` type.

##### `trap_source`

Data type: `Optional[Variant[String, Enum['unset']]]`

Assigns an interface for the source address of all traps. Setting it to 'unset' will revert it to the default value.

Examples:
```
trap_source => 'GigabitEthernet0/3'
```
```
trap_source => 'unset'
```

##### `system_shutdown`

Data type: `Optional[Boolean]`

Enables use of the SNMP reload command.

##### `contact`

Data type: `Optional[Variant[String, Enum['unset']]]`

Sets text for the mib object sysContact. Setting it to 'unset' will revert it to the default value.

Examples:
```
contact => 'SNMP_TEST'
```
```
contact => 'unset'
```

##### `manager`

Data type: `Optional[Boolean]`

When set this value enables the SNMP manager.

##### `manager_session_timeout`

Data type: `Optional[Variant[Integer, Enum['unset']]]`

Modifies the SNMP manager timeout parameter.

Examples:
```
manager_session_timeout => 20
```
```
manager_session_timeout => unset
```

##### `ifmib_ifindex_persist`

Data type: `Optional[Boolean]`

Enables IF-MIB ifindex persistence.

#### Parameters

The following parameters are available in the `ios_snmp_global` type.

##### `name`

namevar

Data type: `String`

ID of the snmp global config. Valid values are default.

Default value: default

### ios_stp_global

Manages the Cisco Spanning-tree Global configuration resource.

#### Properties

The following properties are available in the `ios_stp_global` type.

##### `enable`

Data type: `Optional[Boolean[false]]`

Disable STP functionality by specifying false.

##### `bridge_assurance`

Data type: `Optional[Boolean]`

Bridge Assurance on all network ports

##### `extend_system_id`

Data type: `Optional[Boolean]`

Extend system-id into priority portion of the bridge id (PVST & Rapid PVST only)

##### `loopguard`

Data type: `Optional[Boolean]`

Bridge Assurance on all network ports

##### `mode`

Data type: `Optional[Enum["mst","pvst","rapid-pvst"]]`

Operating Mode

##### `mst_forward_time`

Data type: `Optional[Integer]`

Forward delay for the spanning tree

##### `mst_hello_time`

Data type: `Optional[Integer]`

Hello interval for the spanning tree

##### `mst_inst_vlan_map`

Data type: `Optional[Array[Tuple[Integer,String]]]`

An array of [mst_inst, vlan_range] pairs.

##### `mst_max_age`

Data type: `Optional[Integer[6,40]]`

Max age interval for the spanning tree

##### `mst_max_hops`

Data type: `Optional[Integer[1,255]]`

Max hops value for the spanning tree

##### `mst_name`

Data type: `Optional[String]`

Configuration name.

##### `mst_priority`

Data type: `Optional[Array[Tuple[String,Integer]]]`

An array of [mst_inst_list, priority] pairs.

##### `mst_revision`

Data type: `Optional[Integer]`

Configuration revision number.

##### `pathcost`

Data type: `Optional[Enum["long","short"]]`

Method to calculate default port path cost

##### `portfast`

Data type: `Optional[Array[Enum["default","bpduguard_default","bpdufilter_default"]]]`

Spanning tree portfast options

##### `uplinkfast`

Data type: `Optional[Boolean]`

Enable UplinkFast Feature

##### `uplinkfast_max_update_rate`

Data type: `Optional[Integer]`

Maximum number of update packets per second

##### `vlan_forward_time`

Data type: `Optional[Array[Tuple[String,Integer]]]`

An array of [vlan_inst_list, forward_time] pairs.

##### `vlan_hello_time`

Data type: `Optional[Array[Tuple[String,Integer]]]`

An array of [vlan_inst_list, hello_time] pairs.

##### `vlan_max_age`

Data type: `Optional[Array[Tuple[String,Integer]]]`

An array of [vlan_inst_list, max_age] pairs.

##### `vlan_priority`

Data type: `Optional[Array[Tuple[String,Integer]]]`

An array of [vlan_inst_list, priority] pairs.

#### Parameters

The following parameters are available in the `ios_stp_global` type.

##### `name`

namevar

Data type: `String`

ID of the stp global config. Valid values are default.

Default value: default

## Tasks

### cli_command

Execute CLI Command

**Supports noop?** false

#### Parameters

##### `command`

Data type: `String[1]`

CLI Command to execute

##### `raw`

Data type: `Boolean`

Whether to return the raw output or wrap it into JSON

### config_save

Save running-config to startup-config

**Supports noop?** false

### restore_startup

Copies the startup-config to the running-config

**Supports noop?** false

### backup_config

Backs up the running config from a device to the specified location

#### Parameters

##### `backup_location`

Data type: `String[1]`

Location to save the running config to

##### `raw`

Data type: `Boolean`

Whether to return the raw output or wrap it into JSON

### restore_config

Restores the configuration from a given backup location to the device

#### Parameters

##### `backup_location`

Data type: `String[1]`

Location of the config to restore to the device

##### `raw`

Data type: `Boolean`

Whether to return the raw output or wrap it into JSON
